/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.servicemix.kernel.gshell.features.management;

import java.util.HashMap;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.management.JMException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;

import junit.framework.TestCase;

import org.easymock.EasyMock;

/**
 * Test cases for {@link ManagedFeaturesRegistry}
 */
public class ManagedFeaturesRegistryTest extends TestCase {

    @SuppressWarnings("unchecked")
    public void testInitializeInAfterPropertiesSet() throws Exception {
        final AtomicBoolean registered = new AtomicBoolean(false);
        ManagedFeaturesRegistry registry = new ManagedFeaturesRegistry() {
            @Override
            protected void registerMBeans() throws MalformedObjectNameException, JMException {
                registered.set(true);
            }
        };
        registry.setNamingStrategy(EasyMock.createMock(NamingStrategy.class));
        registry.setManagementAgent(new ManagementAgent());
        registry.registerMBeanServer(EasyMock.createMock(MBeanServer.class), new HashMap());
        
        // the MBeans should only be registered if strategy, agent and server are available
        registry.afterPropertiesSet();
        assertTrue(registered.get());
    }
    
    @SuppressWarnings("unchecked")
    public void testInitializeWhenMBeanServerRegistered() throws Exception {
        final AtomicBoolean registered = new AtomicBoolean(false);
        ManagedFeaturesRegistry registry = new ManagedFeaturesRegistry() {
            @Override
            protected void registerMBeans() throws MalformedObjectNameException, JMException {
                registered.set(true);
            }
        };
        
        MBeanServer server = EasyMock.createMock(MBeanServer.class);
        registry.registerMBeanServer(server, new HashMap());
        // don't register MBeans when no strategy or agent available
        assertFalse("No MBean registration when strategy/agent not set", registered.get());
        
        registry.setManagementAgent(new ManagementAgent());
        registry.setNamingStrategy(EasyMock.createMock(NamingStrategy.class));
        registry.registerMBeanServer(server, new HashMap());
        assertTrue("The MBeans should have been registered", registered.get());
    }
    
    public void testExceptionWhenNoManagementAgentSet() throws Exception {
        ManagedFeaturesRegistry registry = new ManagedFeaturesRegistry();
        try {
            registry.afterPropertiesSet();
            fail("afterPropertiesSet() should have thrown an IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            // this is OK
        }
    }
    
    public void testExceptionWhenNoNamingStrategySet() throws Exception {
        ManagedFeaturesRegistry registry = new ManagedFeaturesRegistry();
        registry.setManagementAgent(new ManagementAgent());
        try {
            registry.afterPropertiesSet();
            fail("afterPropertiesSet() should have thrown an IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            // this is OK
        }
    }
}
